using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Data.OleDb;

namespace BugTracker
{
    public partial class frmMain : Form
    {
        private const string ALL_USERS = "All Users";

        private const string ADD_CAPTION = "Add";
        private const string EDIT_CAPTION = "Edit";
        private const string SAVE_CAPTION = "Save";
        private string sortOrder = "BugID";
        //create the database connection
        OleDbConnection aConnection = new OleDbConnection(
            @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=BugTracker.mdb");

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            LoadDataInCombo(cmbProductList,@"ProductList.txt");
            LoadDataInCombo(cmbProblemGroup,@"ProblemGroup.txt");
            LoadDataInCombo(cmbOwner, @"Owner.txt");
            
            cmbShowForOwner.Items.Add(ALL_USERS);
            cmbShowForOwner.SelectedIndex = 0;
            LoadDataInCombo(cmbShowForOwner, @"Owner.txt");

            LoadDataInCombo(cmbProblemType, @"ProblemType.txt");
            LoadDataInCombo(cmbStatus, @"Status.txt");

            btnAdd.Text = ADD_CAPTION;
            btnUpdate.Text = EDIT_CAPTION;
            grpData.Enabled = false;
            LoadDatabaseInView();
            if (lstUnfixedData.Items.Count > 0)
                lstUnfixedData.Items[0].Selected = true;
        }
        private string iffnvl(object obj)
        {
            if (!(obj.Equals(System.DBNull.Value)))
                return (string)obj;
            else
                return "";
        
        }

        private string isDate(string str)
        {
            if (str==string.Empty)
                return DateTime.Now.ToShortDateString();
            else
                return str;

        }

        private int RunSQLNonQuery(string sqlStatement)
        { 
            //create the command object and store the sql query
            OleDbCommand aCommand = new OleDbCommand(sqlStatement, aConnection);
            int bRet;

            try
            {
                aConnection.Open();

                //create the datareader object to connect to table
                bRet = aCommand.ExecuteNonQuery();

                //close the connection Its important.
                aConnection.Close();
            }
            //Some usual exception handling
            catch(OleDbException e)
            {
                aConnection.Close();
                MessageBox.Show(e.Message);
                return 0;
            }
            return bRet;
        }

        private void LoadDatabaseInView()
        {
            lstUnfixedData.Items.Clear();
            string owner = cmbShowForOwner.Text;
            string sqlStatement;
            if (owner == ALL_USERS)
                sqlStatement = @"Select BugId, BriefDesc, Owner from Bugs where Fixed = 0 AND deleted = 0 order by " + sortOrder;
            else
                sqlStatement = @"Select BugId, BriefDesc, Owner from Bugs where Fixed = 0 AND deleted = 0 AND Owner like '%" + owner + "%' order by " + sortOrder;
            OleDbCommand aCommand = new OleDbCommand(sqlStatement, aConnection);
            try
            {
                aConnection.Open();

                //create the datareader object to connect to table
                OleDbDataReader aReader = aCommand.ExecuteReader();

                //Iterate throuth the database
                while (aReader.Read())
                {
                    ListViewItem lvi = lstUnfixedData.Items.Add(iffnvl(aReader["BugId"]));
                    lvi.SubItems.Add(iffnvl(aReader["BriefDesc"]));
                    lvi.SubItems.Add(iffnvl(aReader["Owner"]));
                }

                //close the reader 
                aReader.Close();

                //close the connection Its important.
                aConnection.Close();
            }
            //Some usual exception handling
            catch (OleDbException e)
            {
                //close the connection Its important.
                aConnection.Close();

                MessageBox.Show(e.Message);
            }  
        }
        private void LoadDataInCombo(ComboBox cmbBox, string filename)
        {
            if(File.Exists(filename))
            {
                string[] productList = File.ReadAllLines(filename);
                for (int i = 0; i < productList.Length; i++)
                {
                    cmbBox.Items.Add(productList[i]);
                }
            }
         }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (btnAdd.Text == ADD_CAPTION)
            {
                btnAdd.Text = SAVE_CAPTION;
                grpData.Enabled = true;

                btnUpdate.Enabled = false;
                btnFixed.Enabled = false;
                btnDelete.Enabled = false;

                grpNavigation.Enabled = false;

                btnCancel.Enabled = true;
            }
            else 
            {
                btnAdd.Text = ADD_CAPTION;
                grpData.Enabled = false;

                btnUpdate.Enabled = true;
                btnFixed.Enabled = true;
                btnDelete.Enabled = true;
                
                grpNavigation.Enabled = true;
                
                btnCancel.Enabled = false;
                
                string sqlStatement;

                sqlStatement = "Insert into bugs(BugId,BriefDesc,ProductList,ProblemGroup,Owner,Problemtype,RecieveDate,Status,StartDate,TargetDate,EndDate,Issues,ActionPoint,Fixed) "
                    + "values('" 
                    + txtID.Text + "','" 
                    + txtDesc.Text + "','"
                    + cmbProductList.Text + "','"
                    + cmbProblemGroup.Text + "','"
                    + cmbOwner.Text + "','"
                    + cmbProblemType.Text + "','"
                    + isDate(txtRecievedDate.Text) + "','"
                    + cmbStatus.Text + "','"
                    + isDate(txtStartDate.Text) + "','"
                    + isDate( txtTargetDate.Text) + "','"
                    + isDate( txtEndDate.Text) + "','"
                    + txtIssues.Text + "','"
                    + txtActionPoint.Text + "',0)";
                RunSQLNonQuery(sqlStatement);
                
                LoadDatabaseInView();

            }
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {
            if (btnUpdate.Text == EDIT_CAPTION)
            {
                btnUpdate.Text = SAVE_CAPTION;
                txtID.Enabled = false;

                grpData.Enabled = true;

                btnAdd.Enabled = false;
                btnFixed.Enabled = false;
                btnDelete.Enabled = false;

                grpNavigation.Enabled = false;

                btnCancel.Enabled = true;
            }
            else
            {
                btnUpdate.Text = EDIT_CAPTION;
                txtID.Enabled = true;

                grpData.Enabled = false;

                btnAdd.Enabled = true;
                btnFixed.Enabled = true;
                btnDelete.Enabled = true;

                grpNavigation.Enabled = true;

                btnCancel.Enabled = false;

                string sqlStatement;

                sqlStatement = "Update bugs set " + 
                    " BriefDesc = '" + txtDesc.Text + "'," +
                    " ProductList = '" +cmbProductList.Text + "',"+
                    " ProblemGroup = '" +cmbProblemGroup.Text + "',"+
                    " Owner = '" +cmbOwner.Text + "',"+
                    " Problemtype = '" +cmbProblemType.Text + "',"+
                    " RecieveDate = '" +isDate(txtRecievedDate.Text) + "',"+
                    " Status = '" +cmbStatus.Text + "',"+
                    " StartDate = '" +isDate(txtStartDate.Text) + "',"+
                    " TargetDate = '" +isDate(txtTargetDate.Text) + "',"+
                    " EndDate = '" +isDate(txtEndDate.Text) + "',"+
                    " Issues = '" +txtIssues.Text + "',"+
                    " ActionPoint = '" +txtActionPoint.Text + "'" + 
                    " where BugId  = '"+txtID.Text+"'";
                
                RunSQLNonQuery(sqlStatement);
                LoadDatabaseInView();
            }
        }

        private void lstUnfixedData_SelectedIndexChanged(object sender, EventArgs e)
        {
            ListView lv = (ListView)sender;
            if(lv.SelectedIndices.Count > 0)
                BrowseData(lv.Items[lv.SelectedIndices[0]].Text);
        }
        private void BrowseData(string BugID)
        {
            string sqlStatement;

            sqlStatement = "Select * from Bugs where BugId = '" + BugID + "'";
            OleDbCommand aCommand = new OleDbCommand(sqlStatement, aConnection);
            try
            {
                aConnection.Open();

                //create the datareader object to connect to table
                OleDbDataReader aReader = aCommand.ExecuteReader();

                //Iterate throuth the database
                while (aReader.Read())
                {
                    txtID.Text = iffnvl(aReader["BugId"]);
                    txtDesc.Text = iffnvl(aReader["BriefDesc"]);
                    
                    cmbOwner.Text = iffnvl(aReader["Owner"]);
                }

                //close the reader 
                aReader.Close();

                //close the connection Its important.
                aConnection.Close();
            }
            //Some usual exception handling
            catch (OleDbException e)
            {
                //close the connection Its important.
                aConnection.Close();

                MessageBox.Show(e.Message);
            }  
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            btnAdd.Text = ADD_CAPTION;
            btnUpdate.Text = EDIT_CAPTION;
            
            grpData.Enabled = false;
            btnCancel.Enabled = false;
            
            btnAdd.Enabled = true;
            btnUpdate.Enabled = true;
            btnFixed.Enabled = true;
            btnDelete.Enabled = true;
            grpNavigation.Enabled = true;
            lstUnfixedData.Enabled = true;
            cmbShowForOwner.Enabled = true;
            txtID.Enabled = true;
        }

        private void btnFixed_Click(object sender, EventArgs e)
        {
            string sqlStatement;

            sqlStatement = "Update bugs set " +
                " fixed = 1" + 
                " where BugId  = '" + txtID.Text + "'";

            RunSQLNonQuery(sqlStatement);
            
            LoadDatabaseInView();

            MessageBox.Show("Unique Number " + txtID.Text +" state is set to fix successfully");
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete " + txtID.Text, "Deletion warning",
              MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) 
            {
                string sqlStatement;

                sqlStatement = "Update bugs set " +
                    " deleted = 1" +
                    " where BugId  = '" + txtID.Text + "'";

                RunSQLNonQuery(sqlStatement);
                LoadDatabaseInView();
            }
        }

        private void cmbShowForOwner_SelectedIndexChanged(object sender, EventArgs e)
        {
            LoadDatabaseInView();
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            LoadDatabaseInView();
        }

        private void lstUnfixedData_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            switch (e.Column)
            { 
                case 0:
                    sortOrder = "BugId";
                    break;
                case 1:
                    sortOrder = "BriefDesc";
                    break;
                case 2:
                    sortOrder = "Owner";
                    break;
                default:
                    sortOrder = "BugId";
                    break;
            }
            LoadDatabaseInView();
        }
    }
}